import sys
import os
from lib.tika.tika import parser
from datetime import date
from lib.pdfminer.pdfminer.pdfparser import PDFParser
from lib.pdfminer.pdfminer.pdfdocument import PDFDocument
os.environ['TIKA_SERVER_JAR'] = 'https://repo1.maven.org/maven2/org/apache/tika/tika-server/1.19/tika-server-1.19.jar'

def replaceNewlines(txt):
    tmp = txt.split("\n")
    res = ""
    for line in tmp :
        if line.replace(" ", "") == "": 
            continue
        if res != "":
            res += '\n'
        res += line
    return res

def reSplit(new, old):
    res = []
    count = 0
    for s in old:
        tmp = ""
        for c in s:
            if c == new[count] :
                tmp += c
                count += 1
        if(tmp != "") :
            res.append(tmp)
    return res

def removeFiligrane(abstract, filigrane):
    if filigrane == "" :
        return abstract
    tmp = abstract.split("\n")
    txt = "".join(tmp).replace(filigrane,"")
    res = reSplit(txt, tmp)
    res = "\n".join(res)
    return res

def getTextFromPDF(fileAddress, filigrane):
    parsed_pdf = parser.from_file(fileAddress)
    data = parsed_pdf['content']
    data = replaceNewlines(data)
    return removeFiligrane(data, filigrane)

def getDate(fileAddress):
    year = str(date.today().year)
    month = "1"
    day = "1"
    with open(fileAddress, 'rb') as fp:
        parser = PDFParser(fp)
        doc = PDFDocument(parser)
        info = doc.info[0]
        tmp = ""
        if "CreationDate" in info:
            tmp = info["CreationDate"].decode('utf-8')
            year = tmp[2:6]
            month = tmp[6:8]
            day = tmp[8:10]

    return year, month, day

def waitAnswer(question):
    answer = input(question + " (Y/N) : ")
    while(answer.lower() != "y" and answer.lower() != "n" ):
        answer = input("Wrong input ! Answer with either y or n : ")
    
    return answer == "y"

def getFiligrane(fileName):
    filigrane = ""
    question = "Does the file \"" + fileName + "\" has a filigrane ?"
    if waitAnswer(question):
        filigrane = input("Please enter the filigrane : ")
    return filigrane

def PDFtoTXT(fileAddress):
    fileName = fileAddress.replace("pdf/", "")
    year, month, day = getDate(fileAddress)
    filigrane = getFiligrane(fileName)
    txt = "#Date from PDF# : " + month + " " + day + " " + year + "\n\n"
    txt += getTextFromPDF(fileAddress, filigrane)

    with open("txt/" + fileName.replace(".pdf",".txt"), "w", encoding='utf-8-sig') as file:
        file.write(txt)

    print("Your file has been converted successfully.")

def main():

    if len(sys.argv) == 2:
        fileName = sys.argv[1]
        PDFtoTXT("pdf/" + fileName)

    return 

if __name__ == '__main__':
    sys.exit(main()) 