# coding: utf-8

import sys
import os
import re
from datetime import date
from lib.langdetect.langdetect import detect
from lib.langdetect.langdetect.lang_detect_exception import LangDetectException

def detectLanguages(abstract, languages):
    if re.search('[a-zA-Z]', abstract) == None:
        return languages
    try :
        tmp = detect(abstract)
        if tmp in languages :
            languages[tmp] += 1
        else:
            languages[tmp] = 1
    except LangDetectException:
        pass
    return languages

def estimateLanguagesPercentage(languages):
    total = 0
    max = 0
    principal = ""
    res = {}
    for l in languages:
        total += int(languages[l])
    for l in languages:
        tmp = int(languages[l]) / total * 100
        if tmp > max :
            max = tmp
            principal = l
        res[l] = tmp
    return res, principal

def detectMultipleLanguages(languages, source):
    languages, principal = estimateLanguagesPercentage(languages)
    detected = ""
    for l in languages :
        if languages[l] >= 15:
            if detected != "" :
                detected += "| "
            detected += l + " : " + str(languages[l]) + "%"
    if len(detected) > 2:
        print("Warning ! Multiple languages have been detected in the pdf \"" + source + "\" !")
        print("The following languages have been detected : " + detected)
        print("This could massively affect the analysis of Gargantext.")
        print("You can translate it with the TsvTranslator tool.")

def segmentAbstract(abstract, tsv, author, source, year, month, day):
    nbLines = len(abstract)
    n = 0
    count = 1
    languages = {}
    while n < nbLines - 2:
        doc = "".join(abstract[n: n + 9]).replace("�", "")
        title = source + " : Part " + str(count)
        tsv += correctedSequence(author, False) + "\t" + correctedSequence(source, False) + "\t" + year + "\t"+ month + "\t" + day + "\t"
        tsv += correctedSequence(title, False) + "\t"
        tsv +=  correctedSequence(doc, True)
        if tsv[-1] != "\n":
            tsv += "\n"
        n += 7
        count += 1
        if n > nbLines - 9 and n != nbLines - 2:
            n = nbLines - 9
        languages = detectLanguages(doc, languages)
    
    detectMultipleLanguages(languages, source)
    return tsv

def waitAnswer(question):
    answer = input(question + " (Y/N) : ")
    while(answer.lower() != "y" and answer.lower() != "n" ):
        answer = input("Wrong input ! Answer with either y or n : ")
    
    return answer == "y"


def getAuthorAndTitle(fileName):

    author = ""
    title = fileName.replace(".pdf", "")
    question = "The author isn't set for the file \"" + fileName + "\". Do you want to set it ?"
    if waitAnswer(question):
        author = input("Please enter the author(s) name : ")
    
    question = "Do you want to change the title for \"" + fileName + "\" ?"
    if waitAnswer(question):
        title = input("Please enter the title : ")
    return author, title

def correctedSequence(text, last):
    tmp = text.replace("\"", "\"\"")
    find = "\t" in text or "\"" in text or "\n" in text
    if find:
        if text[len(text) - 1] == "\n" :
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
        if last :
            tmp += "\n"
    return tmp

def getDate(file):
    year = str(date.today().year)
    month = "1"
    day = "1"
    if "#Date from PDF# : " in file[0] :
        tmp = file[0].replace("#Date from PDF# : ", "")
        month = tmp[0:2]
        day = tmp[3:5]
        year = tmp[6:10]
        file = file[2:]
    return year, month, day,file

def getTxt(fileAddress):
    file2 = []
    with open(fileAddress,"r", encoding='utf-8-sig') as file:
        for line in file:
            file2.append(line)
    return getDate(file2) 

def txtToTSV(fileAddress):

    fileName = fileAddress.replace("txt/", "")
    year, month, day, abstract = getTxt(fileAddress)
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    author, source = getAuthorAndTitle(fileName)
    tsv = segmentAbstract(abstract, tsv, author, source, year, month, day)
    
    with open("tsv/" + fileName.replace(".txt", ".tsv") ,"w", encoding="utf-8-sig") as file:
        file.write(tsv)

    print("Your file has been converted successfully.")

def main():

    if len(sys.argv) == 2:
        fileName = sys.argv[1]
        directoryName = sys.argv[1]
        txtToTSV("txt/" + fileName)

    return 

if __name__ == '__main__':
    sys.exit(main()) 