# coding: utf-8

import sys
import os
import csv
import re
from lib.googletrans.googletrans import Translator as GoogleTranslator
from lib.googletrans.googletrans import LANGUAGES
from lib.langdetect.langdetect import detect
from lib.langdetect.langdetect.lang_detect_exception import LangDetectException


def detectLanguages(abstract, languages, translator):
    if re.search('[a-zA-Z]', abstract) == None:
        return languages
    try:
        tmp = detect(abstract)
        if tmp in languages:
            languages[tmp] += 1
        else:
            languages[tmp] = 1
    except LangDetectException:
        pass
    return languages


def estimateLanguagesPercentage(languages):
    total = 0
    max = 0
    principal = ""
    res = {}
    for l in languages:
        total += int(languages[l])
    for l in languages:
        tmp = int(languages[l]) / total * 100
        if tmp > max:
            max = tmp
            principal = l
        if tmp >= 15:
            res[l] = tmp
    return res, principal


def waitAnswer(question):
    answer = input(question + " (Y/N) : ")
    while (answer.lower() != "y" and answer.lower() != "n"):
        answer = input("Wrong input ! Answer with either y or n : ")

    return answer == "y"


def otherPossibilities(srcLang, destLang):
    question = "Do you want to choose the source language and the translation language ?"
    if waitAnswer(question):
        srcLang = input("Please enter the source language : ")
        while srcLang not in LANGUAGES:
            srcLang = input(
                "This is not a valide language. Please enter it again : ")
        destLang = input("Please enter the translation language : ")
        while destLang not in LANGUAGES:
            destLang = input(
                "This is not a valide language. Please enter it again : ")
    return srcLang, destLang


def translatePossibility(languages, principal):
    srcLang = principal
    destLang = principal
    if len(languages) == 1:
        print("Only one language has been detected : \"" + principal + "\"")
        srcLang, destLang = otherPossibilities(srcLang, destLang)
    if len(languages) == 2:
        second = ""
        for l in languages:
            if l != principal:
                second = l
        print("2 differents languages have been detected : ")
        print(principal + " at " + str(languages[principal]) +
              "% and " + second + " at " + str(languages[second]) + "%")
        question = "Do you want to translate from " + second + " to " + principal + " ?"
        if waitAnswer(question):
            srcLang, destLang = second, principal
        else:
            srcLang, destLang = otherPossibilities(srcLang, destLang)
    if len(languages) > 2:
        print("There are more than 2 languages detected ! The text can't be translated.")
        print("Following languages have been detected : " + str(languages))
    return srcLang, destLang


def inspectLanguages(fileName, separator):
    translator = GoogleTranslator()
    languages = {}
    with open(fileName, 'r') as file:
        reader = csv.DictReader(file, delimiter=separator)
        for row in reader:
            for name, value in row.items():
                if name.lower() == 'abstract':
                    languages = detectLanguages(value, languages, translator)

    languages, principal = estimateLanguagesPercentage(languages)
    return translatePossibility(languages, principal)


def translate(text, srcLang, destLang):
    translator = GoogleTranslator()
    return translator.translate(text, src=srcLang, dest=destLang).text


def correctedSequence(text, separator):
    tmp = text.replace("\"", "\"\"")
    find = separator in text or "\"" in text or "\n" in text
    if find:
        if text[len(text) - 1] == "\n":
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
    return tmp


def getSeparator(fileName):

    with open(fileName, "r", encoding='utf-8-sig') as file:
        line = file.readline()

        if ',' in line:
            return ','

        if ';' in line:
            return ';'

    return '\t'


def getColumnsNames(fileName, separator):
    data = ""
    total = 1
    with open(fileName, 'r') as file:
        reader = csv.DictReader(file, delimiter=separator)
        columnsNames = []
        for row in reader:

            for name, value in row.items():
                if data != "":
                    data += "\t"
                data += name.replace("\ufeff", "")
            break
        data += "\n"

        total += sum(1 for row in reader)
    return data, total


def getContent(fileName, separator, data, srcLang, destLang, total):
    with open(fileName, 'r') as file:
        percent = -1
        reader = csv.DictReader(file, delimiter=separator)
        count = 1
        for row in reader:
            tmp = ""
            first = True
            loading = int(count / total * 100)
            if loading % 10 == 0 and loading != percent:
                print("Loading : " + str(loading) + "% done ...")
                percent = loading
            for name, value in row.items():
                if not first:
                    tmp += "\t"
                else:
                    first = False
                if name.lower() == "abstract" and re.search('[a-zA-Z]', value) != None:
                    tmp += correctedSequence(translate(value,
                                             srcLang, destLang), separator)
                else:
                    tmp += correctedSequence(value, separator)
            count += 1
            data += tmp + "\n"
    return data


def translateTSV(fileName):

    separator = getSeparator(fileName)
    srcLang, destLang = inspectLanguages(fileName, separator)
    if srcLang == destLang:
        return
    data, total = getColumnsNames(fileName, separator)
    data = getContent(fileName, separator, data, srcLang, destLang, total)

    with open("translatedTSV/" + fileName.replace("tsv/", ""), "w", encoding='utf-8-sig') as file:
        file.write(data)


def main():

    if len(sys.argv) == 2:

        fileName = sys.argv[1]
        translateTSV("tsv/" + fileName)
    return


if __name__ == '__main__':
    sys.exit(main())
