# coding: utf-8

import sys
import os
from datetime import date
from youtubesearchpython import VideosSearch
from lib.youtubetranscript.youtube_transcript_api import YouTubeTranscriptApi
from lib.youtubetranscript.youtube_transcript_api._transcripts import NoTranscriptFound
from lib.youtubetranscript.youtube_transcript_api._transcripts import TranscriptsDisabled


def ytbSearch(search, n):
    videosSearch = VideosSearch(search, limit=n)
    result = videosSearch.result()["result"]
    videos = []
    while len(videos) < n:
        for video in result:
            id = video["id"]
            title = video["title"]
            author = video["channel"]["name"]
            videos.append([id, author, title])
            if len(videos) == n:
                break
        search.next()
        result = search.result()["result"]
    return videos


def getLang(list):
    for lang in list:
        return str(lang).split(" ")[0]


def translatedTranscript(lang, lst, title, manual):
    if lang != "en":
        res = lst.find_transcript([lang])
        trans = res.translate("en").fetch()
        print("Subtitles are translated from " +
              lang + " to english for video : " + title + "\n")
        return trans
    if manual:
        print("Subtitles are manually created from " +
              lang + " for video : " + title + "\n")
    else:
        print("Subtitles are generated from " +
              lang + " for video : " + title + "\n")
    return lst.find_transcript([lang]).fetch()


def ytbTranscript(id, title):
    try:
        transcriptList = YouTubeTranscriptApi.list_transcripts(id)
        lang = getLang(transcriptList)
        try:
            transcriptList.find_manually_created_transcript([lang])
            return translatedTranscript(lang, transcriptList, title, True), True
        except NoTranscriptFound:
            return translatedTranscript(lang, transcriptList, title, False), False
    except TranscriptsDisabled:
        print("Subtitles disabled for video : " + title + "\n")
        return None, False


def tsvAdd(tsv, abstract, author, title, date, count):
    part = title + " : Part " + str(count)
    tsv += correctedSequence(author, False) + "\t" + correctedSequence(
        title, False) + "\t" + date + "\t" + "1" + "\t" + "1" + "\t"
    tsv += correctedSequence(part, False) + "\t"
    tsv += correctedSequence(abstract, True)
    return tsv


def transcriptAutomaticToDoc(transcript, author, title, date):
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    time = 0
    count = 1
    tmp = ""
    for part in transcript:
        tmp += part["text"] + "\n"
        time += int(part["duration"])
        if time >= 20:
            tsv = tsvAdd(tsv, tmp, author, title, date, count)
            tmp = ""
            time = 0
            count += 1
    with open("tsv/" + title + ".tsv", "w", encoding='utf-8-sig') as file:
        file.write(tsv)
    tsv = "\n".join(tsv.split("\n")[1:])
    return tsv


def detectPoint(text):
    if "." in text:
        tmp2 = text.split(".")
        return True, ".", tmp2
    elif "!" in text:
        tmp2 = text.split("!")
        return True, "!", tmp2
    elif "?" in text:
        tmp2 = text.split("?")
        return True, "?", tmp2
    return False, "", ""


def transcriptManualToDoc(transcript, author, title, date):
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    time = 0
    count = 1
    tmp = ""
    end = False
    for part in transcript:
        text = part["text"]
        split, end, tmp2 = detectPoint(text)
        if split:
            if time >= 10:
                tmp += tmp2[0] + end
                tsv = tsvAdd(tsv, tmp, author, title, date, count)
                tmp = "".join(tmp2[1:]) + " "
                time = 0
                count += 1
            else:
                tmp += text
                time += int(part["duration"])
        else:
            tmp += text + "\n"
            time += int(part["duration"])
    with open("tsv/" + title + ".tsv", "w", encoding='utf-8-sig') as file:
        file.write(tsv)
    tsv = "\n".join(tsv.split("\n")[1:])
    return tsv


def correctedSequence(text, last):
    tmp = text.replace("\"", "\"\"")
    find = "\t" in text or "\"" in text or "\n" in text
    if find:
        if text[len(text) - 1] == "\n":
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
    if last:
        tmp += "\n"
    return tmp


def transcriptToTsv(search, nbVideos):
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    videos = ytbSearch(search, nbVideos * 4)
    for video in videos:
        id, author, title = video[0], video[1], video[2]
        print("Processing conversion for video : " + title)
        transcript, manual = ytbTranscript(id, title)
        if transcript == None:
            continue
        if manual:
            tsv += transcriptManualToDoc(transcript,
                                         author, title, str(date.today().year))
        else:
            tsv += transcriptAutomaticToDoc(transcript,
                                            author, title, str(date.today().year))
    with open("tsv/final.tsv", "w", encoding='utf-8-sig') as file:
        file.write(tsv)


def main():

    search = input("Enter the subject of the videos you want to search : ")

    nbVideos = input("Enter the number of videos you want to search : ")
    while not nbVideos.isnumeric():
        nbVideos = input("Valeur incorrecte ! Veuillez entrez un nombre : ")
    nbVideos = int(nbVideos)
    transcriptToTsv(search, nbVideos)

    return


if __name__ == '__main__':
    sys.exit(main())
